# -*- coding: utf-8 -*-
"""Examples of how to run work chains."""
from __future__ import absolute_import
from pathlib import Path

THIS_DIR = Path(__file__).resolve().parent
DATA_DIR = THIS_DIR / 'data'
